#ifndef PYTHONIC_INCLUDE_OPERATOR_IRSHIFT_HPP
#define PYTHONIC_INCLUDE_OPERATOR_IRSHIFT_HPP

#define OPERATOR_NAME irshift
#define OPERATOR_SYMBOL >>
#define OPERATOR_ISYMBOL >>=
#include "pythonic/include/operator_/icommon.hpp"

#endif
