#ifndef PYTHONIC_INCLUDE_OPERATOR_ADD__HPP
#define PYTHONIC_INCLUDE_OPERATOR_ADD__HPP

#include "pythonic/include/utils/functor.hpp"
#include "pythonic/include/operator_/add.hpp"

PYTHONIC_NS_BEGIN

namespace operator_
{
  USING_FUNCTOR(__add__, add);
}
PYTHONIC_NS_END

#endif
